@@ -8,22 +8,14 @@ using namespace frc2;
 
 InstantCommand::InstantCommand(std::function<void()> toRun,
                                std::initializer_list<std::shared_ptr<Subsystem>> requirements)
-    : m_toRun{std::move(toRun)} {
-  AddRequirements(requirements);
-}
+    : FunctionalCommand(
+          std::move(toRun), [] {}, [](bool interrupted) {}, [] { return true; },
+          requirements) {}
 
 InstantCommand::InstantCommand(std::function<void()> toRun,
-                               wpi::span<std::shared_ptr<Subsystem>> requirements)
-    : m_toRun{std::move(toRun)} {
-  AddRequirements(requirements);
-}
+                               std::span<std::shared_ptr<Subsystem>> requirements)
+    : FunctionalCommand(
+          std::move(toRun), [] {}, [](bool interrupted) {}, [] { return true; },
+          requirements) {}
 
-InstantCommand::InstantCommand() : m_toRun{[] {}} {}
-
-void InstantCommand::Initialize() {
-  m_toRun();
-}
-
-bool InstantCommand::IsFinished() {
-  return true;
-}
+InstantCommand::InstantCommand() : InstantCommand([] {}) {}
