@@ -6,8 +6,10 @@
 
 #include <functional>
 #include <initializer_list>
+#include <memory>
 #include <span>
 #include <string>
+#include <type_traits>
 #include <utility>
 #include <vector>
 
