@@ -6,10 +6,10 @@
 
 #include <functional>
 #include <initializer_list>
+#include <span>
 
 #include <frc/Notifier.h>
 #include <units/time.h>
-#include <wpi/span.h>
 
 #include "frc2/command/CommandBase.h"
 #include "frc2/command/CommandHelper.h"
@@ -18,7 +18,7 @@ namespace frc2 {
 /**
  * A command that starts a notifier to run the given runnable periodically in a
  * separate thread. Has no end condition as-is; either subclass it or use
- * Command::WithTimeout(double) or Command::WithInterrupt(BooleanSupplier) to
+ * Command::WithTimeout(double) or Command::Until(BooleanSupplier) to
  * give it one.
  *
  * <p>WARNING: Do not use this class unless you are confident in your ability to
@@ -45,7 +45,7 @@ class NotifierCommand : public CommandBase {
    * @param requirements the subsystems required by this command
    */
   NotifierCommand(std::function<void()> toRun, units::second_t period,
-                  wpi::span<std::shared_ptr<Subsystem>> requirements = {});
+                  std::span<std::shared_ptr<Subsystem>> requirements = {});
 
   NotifierCommand(NotifierCommand&& other);
 
