@@ -6,9 +6,9 @@
 
 #include <functional>
 #include <initializer_list>
+#include <span>
 
 #include <frc/controller/PIDController.h>
-#include <wpi/span.h>
 
 #include "frc2/command/CommandBase.h"
 #include "frc2/command/CommandHelper.h"
@@ -54,7 +54,7 @@ class PIDCommand : public CommandBase {
              std::function<double()> measurementSource,
              std::function<double()> setpointSource,
              std::function<void(double)> useOutput,
-             wpi::span<std::shared_ptr<Subsystem>> requirements = {});
+             std::span<std::shared_ptr<Subsystem>> requirements = {});
 
   /**
    * Creates a new PIDCommand, which controls the given output with a
@@ -84,7 +84,7 @@ class PIDCommand : public CommandBase {
   PIDCommand(PIDController controller,
              std::function<double()> measurementSource, double setpoint,
              std::function<void(double)> useOutput,
-             wpi::span<std::shared_ptr<Subsystem>> requirements = {});
+             std::span<std::shared_ptr<Subsystem>> requirements = {});
 
   PIDCommand(PIDCommand&& other) = default;
 
