@@ -61,10 +61,7 @@ public:
     this->AddRequirements(requirements);
   }
 
-  void Initialize() override {
-    m_timer.Reset();
-    m_timer.Start();
-  }
+  void Initialize() override { m_timer.Restart(); }
 
   void Execute() override { m_output(m_profile.Calculate(m_timer.Get())); }
 
